import numpy as np
import pytest
import kdtree
from sklearn.neighbors import KDTree


@pytest.mark.parametrize("n", [10, 100, 1000, 10000])
@pytest.mark.parametrize("d", [10, 100])
@pytest.mark.parametrize("leaf_size", [256, 1024])
def test_creation_deletion(n, d, leaf_size):
    np.random.seed(1)
    X = np.random.rand(n, d)

    tree = kdtree.KDTree(X, leaf_size=256)
    del tree

@pytest.mark.parametrize("n", [10, 100, 1000, 10000])
@pytest.mark.parametrize("d", [10, 100])
@pytest.mark.parametrize("k", [1, 2, 5, 10])
@pytest.mark.parametrize("leaf_size", [256, 1024])
def test_against_sklearn(n, d, k, leaf_size, n_query=1):
    np.random.seed(2)
    X = np.random.rand(n, d)
    query_points = np.random.rand(n_query, d)

    tree = kdtree.KDTree(X, leaf_size=256)
    skl_tree = KDTree(X, leaf_size=256)

    knn_indices = np.zeros((n_query, k), dtype=np.int32)
    knn_distances = np.zeros((n_query, k), dtype=np.float64)
    tree.query(query_points, knn_indices, knn_distances)
    skl_knn_distances, skl_knn_indices = skl_tree.query(
        query_points,
        k=k,
        return_distance=True
    )

    # Adapting types
    skl_knn_indices = skl_knn_indices.astype(np.int32)

    np.testing.assert_equal(knn_indices, skl_knn_indices)
    # np.testing.assert_almost_equal(knn_distances, skl_knn_distances)
